/*
 * Copyright (c) Jupyter Development Team.
 * Distributed under the terms of the Modified BSD License.
 */
.urth-toc {
    position: fixed;
    bottom: 0px;
    left: 0px;
    width: 200px;
    padding: 5px 5px 15px 5px;
}

.urth-toc-visible {
    border-top: 1px solid #ddd;
    border-right: 1px solid #ddd;
    background-color: rgba(255, 255, 255, 0.9);
    z-index: 1000;
}

.urth-toc ul {
    margin: 0;
    padding: 0;
}

.urth-toc li {
    font-weight: 400;
    line-height: 1.2em;
    list-style-type: none;
    margin: 0;
    padding: 0;
    color: #1598cb;
    text-overflow: ellipsis;
    white-space: nowrap;
    overflow: hidden;
}

.urth-toc .toggle {
    position: absolute;
    bottom: 0px;
    left: 0px;
    opacity: 0.6;
    text-decoration: none;
}

.urth-toc .toggle:hover {
    opacity: 1.0;
}

.urth-toc li.urth-h1 {
    margin-left: 0;
}
.urth-toc li.urth-h2 {
    margin-left: 10px;
}
.urth-toc li.urth-h3 {
    margin-left: 20px;
}
.urth-toc li.urth-h4 {
    margin-left: 30px;
}
.urth-toc a {
    color: #1598cb;
}
.urth-toc a:hover {
    color: #1FAFE7;
}
.urth-toc li:hover {
    text-decoration: underline;
}